@@ -151,7 +151,7 @@ use Mojo::IOLoop;
 use Mojo::Log;
 use utf8;
 
-our $VERSION = 4.01;
+our $VERSION = 4.02;
 
 require Exporter;
 our @ISA = qw(Exporter);
@@ -995,7 +995,10 @@ sub serve_changes {
     sub {
       return unless $_ = decode_utf8($fh->readline);
       chomp;
-      split(/\x1f/), $host, $space, 0 });
+      split(/\x1f/), $host, $space, 0 },
+    undef, #$kept
+    undef, #$filter
+    $style);
   $stream->write("\n");
   print_link($stream, $host, $space, "More...", "do/changes/" . 10 * $n . ($style ? "/$style" : ""));
 }
@@ -1033,8 +1036,9 @@ sub serve_all_changes {
     sub { print_link($stream, @_) },
     sub { $stream->write(encode_utf8 join("\n", @_, "")) },
     sub { @{shift(@$log) }, 1 if @$log },
-    undef,
-    $filter);
+    undef, # $kept
+    $filter,
+    $style);
   $stream->write("\n");
   print_link($stream, $host, undef, "More...", "do/all/changes/" . 10 * $n . ($style ? "/$style" : ""));
 }
@@ -1047,7 +1051,7 @@ sub all_logs {
   # merge all logs
   my @log;
   my $dir = $server->{wiki_dir};
-  my @spaces = space_dirs($stream);
+  my @spaces = space_dirs();
   for my $space (@spaces) {
     my $changes = $dir;
     $changes .= "/$space" if $space;
@@ -1101,7 +1105,8 @@ sub read_log {
 # implementation checks for the existence of the keep file. $filter describes
 # how changes are to be filtered: 'latest' means that only the latest change
 # will be shown, i.e. a link to current revision. The default is to show all
-# changes.
+# changes. $style is "coloured" or "fancy" or undefined to indicate what sort of
+# changes we are looking at.
 sub changes {
   my $stream = shift;
   my $n = shift;
@@ -1115,6 +1120,7 @@ sub changes {
     -e wiki_dir($host, $space) . "/keep/$id/$revision.gmi";
   };
   my $filter = shift||'';
+  my $style = shift;
   my $last_day = '';
   my %seen;
   for (1 .. $n) {
@@ -1131,7 +1137,7 @@ sub changes {
     if ($revision eq "🖹") {
       # a deleted page
       $link->($host, $space, "$name (deleted)", "page/$id");
-      $link->($host, $space, "History", "history/$id");
+      $link->($host, $space, "History", "history/$id" . ($style ? "/10/$style" : ""));
       $seen{$name} = 1;
     } elsif ($revision eq "🖻") {
       # a deleted file
@@ -1140,15 +1146,17 @@ sub changes {
     } elsif ($revision > 0) {
       # a page
       if ($seen{$name}) {
-	$link->($host, $space, "$name ($revision)", "page/$id/$revision");;
-	$link->($host, $space, "Differences", "diff/$id/$revision") if $kept->($host, $space, $id, $revision);
+	$link->($host, $space, "$name ($revision)", "page/$id/$revision");
+	# there is no fancy diff, just colour diff
+	$link->($host, $space, "Differences", "diff/$id/$revision" . ($style ? "/colour" : ""))
+	    if $kept->($host, $space, $id, $revision);
       } elsif ($filter eq "latest") {
 	$link->($host, $space, "$name", "page/$id");
 	$link->($host, $space, "History", "history/$id");
 	$seen{$name} = 1;
       } else {
 	$link->($host, $space, "$name (current)", "page/$id");
-	$link->($host, $space, "History", "history/$id");
+	$link->($host, $space, "History", "history/$id" . ($style ? "/10/$style" : ""));
 	$seen{$name} = 1;
       }
     } else {
@@ -1716,7 +1724,10 @@ sub serve_history {
       chomp;
       my ($ts, $id_log, $revision, $code) = split(/\x1f/);
       goto READ if $id_log ne $id;
-      $ts, $id_log, $revision, $code, $host, $space, 0 });
+      $ts, $id_log, $revision, $code, $host, $space, 0 },
+    undef, #$kept
+    undef, #$filter
+    $style);
   $stream->write("\n");
   print_link($stream, $host, $space, "More...", "history/" . uri_escape_utf8($id) . "/" . 10 * $n . ($style ? "/$style" : ""));
 }
@@ -1936,7 +1947,6 @@ sub space {
 }
 
 sub space_dirs {
-  my $stream = shift;
   my @spaces;
   if (keys %{$server->{host}} > 1) {
     push @spaces, keys %{$server->{host}};
@@ -1984,7 +1994,7 @@ sub is_upload {
     my $host = $1;
     my($scheme, $authority, $path, $query, $fragment) =
 	$request =~ m|(?:([^:/?#]+):)?(?://([^/?#]*))?([^?#]*)(?:\?([^#]*))?(?:#(.*))?|;
-    if ($path =~ m!^(?:/($spaces_regex))?(?:/raw)?/([^/;=&]+(?:;\w+=[^;=&]+)+)!) {
+    if ($path =~ m!^(?:/($spaces_regex))?(?:/raw|/page)?/([^/;=&]+(?:;\w+=[^;=&]+)+)!) {
       my $space = $1;
       my ($id, @params) = split(/[;=&]/, $2);
       my $params = { map {decode_utf8(uri_unescape($_))} @params };
